# 根据 CVE-ID 从 CVE 获取漏洞信息
import httpx
from bs4 import BeautifulSoup
from summer_modules.web_request_utils import getUserAgent
import json
from summer_modules.utils import write_dict_to_json_file
from pathlib import Path
from summer_modules.ai.deepseek import DeepseekClient
from . import CVE_LOGGER, CVE_INFO_BASE_DIR, CURRENT_DIR


class CVEInfo:

    def __init__(self, deepseek_apikey: str):
        self.deepseek_client = DeepseekClient(deepseek_apikey)

    def _get_vul_info_from_cve(self, cve_id, client, headers):
        resp = client.get(
            f"https://cveawg.mitre.org/api/cve/{cve_id}",
            # f"https://cve.mitre.org/cgi-bin/cvename.cgi?name={cve_id}",
            headers=headers,
        )
        if resp.status_code != 200:
            CVE_LOGGER.error(
                f"获取漏洞信息失败: {resp.text}, cve_id: {cve_id}, status_code: {resp.status_code}"
            )
            return None
        response_data_json = resp.json()
        cve_info = response_data_json

        descriptions = {}
        for item in cve_info["containers"]["cna"]["descriptions"]:
            if item["lang"] in ["en", "en-US"]:
                descriptions["en"] = item["value"]
            else:
                CVE_LOGGER.info(f"未知的语言: {item['lang']}")
                descriptions[item["lang"]] = item["value"]

        # 调用 deepseek 翻译 descriptions["en"] 的内容新增字段 descriptions["cn"]
        descriptions["cn"] = self.deepseek_client.translate_text(descriptions["en"])
        cve_info["containers"]["cna"]["descriptions"] = descriptions

        cve_info_base_dir = CVE_INFO_BASE_DIR / cve_id
        cve_info_base_dir.mkdir(parents=True, exist_ok=True)
        write_dict_to_json_file(cve_info, cve_info_base_dir / f"{cve_id}_cve.json")
        CVE_LOGGER.info(f"获取漏洞信息成功: {cve_id}")
        return cve_info

    def get_cve_info(self, cve_id: str, enable_local_search=True) -> dict:
        """根据 CVE-ID 从 CVE 获取漏洞信息
        Args:
            cve_id (str): CVE-ID
            enable_local_search (bool, optional): 是否启用本地搜索. Defaults to True.
        Returns:
            dict: 漏洞信息
        """
        if enable_local_search:
            cve_info_cve_dir = CVE_INFO_BASE_DIR / cve_id
            cve_info_cve_filepath = cve_info_cve_dir / f"{cve_id}_cve.json"
            if cve_info_cve_filepath.exists():
                CVE_LOGGER.debug(f"本地已存在漏洞信息: {cve_id}，从本地读取")
                with open(cve_info_cve_filepath, "r") as f:
                    return json.load(f)
        UA = getUserAgent()
        headers = {
            "user-agent": UA,
            "accept": "application/json, text/plain, */*",
            "accept-encoding": "gzip, deflate, br, zstd",
            "accept-language": "zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6",
            "connection": "keep-alive",
            "dnt": "1",
            "host": "cve.mitre.org",
        }
        try:
            with httpx.Client(verify=False, timeout=20) as client:
                return self._get_vul_info_from_cve(cve_id, client, headers)
        except Exception as e:
            CVE_LOGGER.error(f"获取漏洞信息失败: cve_id: {cve_id}, error: {e}")
            return None

    def get_cve_description(self, cve_id: str, lang="cn") -> str:
        """根据 CVE-ID 获取漏洞描述

        Args:
            cve_id (str): CVE ID, 如 "CVE-2021-44228"
            lang (str): 返回描述的语言，支持 "cn" 或 "en"

        Returns:
            str: 漏洞描述，获取失败时返回空字符串
        """

        # 判断 CVE-ID 是否是 CVE-YYYY-NNNN 格式
        if not cve_id.startswith("CVE-"):
            CVE_LOGGER.warning(f"CVE-ID 不是标准格式：{cve_id}, 请检查")
            return ""
        # 去除首尾空格
        cve_id = cve_id.strip()

        def extract_description(info, language):
            """从 CVE 信息中提取指定语言的描述"""
            if language not in ["en", "cn"]:
                CVE_LOGGER.error(f"不支持的语言: {language}")
                return ""

            try:
                return info["containers"]["cna"]["descriptions"][language]
            except (KeyError, TypeError) as e:
                CVE_LOGGER.error(
                    f"提取漏洞描述失败: {cve_id}, 语言: {language}, 错误: {e}"
                )
                return ""

        cve_info = self.get_cve_info(cve_id)
        if cve_info:
            return extract_description(cve_info, lang)

        CVE_LOGGER.error(f"无法获取漏洞信息: {cve_id}")
        return ""

    def test_get_cve_info_from_cve(self):
        cve_id = "CVE-2021-44228"
        cve_info = self.get_cve_info(cve_id)
        CVE_LOGGER.info(f"{cve_id}-cve_info: {cve_info}")

    def test_get_cve_description(self):
        cve_id = "CVE-2021-44228"
        cve_description = self.get_cve_description(cve_id)
        CVE_LOGGER.info(f"{cve_id}-cve_description: {cve_description}")
